@@ -10,10 +10,18 @@
 #include <list>
 #include <vector>
 
+namespace JDLIB
+{
+    class ConfLoader;
+}
+
+
 namespace CONFIG
 {
     class ConfigItems
     {
+        // 設定をロード済み
+        bool m_loaded;
 
     public:
 
@@ -22,6 +30,9 @@ namespace CONFIG
         bool restore_article;
         bool restore_image;
 
+        // 自前でウィンドウ配置を管理する
+        bool manage_winpos;
+
         // フォント
         std::vector< std::string > fontname;
 
@@ -30,6 +41,7 @@ namespace CONFIG
 
         // ref_prefix の後のスペースの数
         int ref_prefix_space;
+        std::string ref_prefix_space_str;
 
         // キャッシュのルートディレクトリ
         // 旧バージョンとの互換のため残しているだけで使用していない
@@ -39,16 +51,19 @@ namespace CONFIG
         bool use_proxy_for2ch;
         std::string proxy_for2ch;
         int proxy_port_for2ch;
+        std::string proxy_basicauth_for2ch;
 
         // 書き込み用プロクシとポート番号
         bool use_proxy_for2ch_w;
         std::string proxy_for2ch_w;
         int proxy_port_for2ch_w;
+        std::string proxy_basicauth_for2ch_w;
 
         // 2chの外にアクセスするときのプロクシとポート番号
         bool use_proxy_for_data;
         std::string proxy_for_data;
         int proxy_port_for_data;
+        std::string proxy_basicauth_for_data;
 
         // 2ch にアクセスするときのエージェント名
         std::string agent_for2ch;
@@ -66,6 +81,7 @@ namespace CONFIG
         int loader_timeout;
         int loader_timeout_post;
         int loader_timeout_img;
+        int loader_timeout_checkupdate;
 
         // ipv6使用
         bool use_ipv6;
@@ -89,24 +105,41 @@ namespace CONFIG
         int imgpopup_width;
         int imgpopup_height;
 
+        // 画像ポップアップを使用する
+        bool use_image_popup;
+
         // 画像ビューを使用する
         bool use_image_view;
 
         // インライン画像を表示する
         bool use_inline_image;
 
+        // ssspアイコン表示
+        bool show_ssspicon;
+
         // 画像にモザイクかける
         bool use_mosaic;
 
+        // モザイクの大きさ
+        // 画像を一度mosaic_sizeまで縮めてから表示する
+        // 画像のサイズがmosaic_sizeより小さい場合はモザイクをかけない
+        int mosaic_size;
+
         // 画像をデフォルトでウィンドウサイズに合わせる
         bool zoom_to_fit;
 
         // 画像キャッシュ削除の日数
         int del_img_day;
 
-        // ダウンロードする画像の最大サイズ(Mbyte)        
+        // 画像あぼーん削除の日数
+        int del_imgabone_day;
+
+        // ダウンロードする画像の最大ファイルサイズ(Mbyte)        
         int max_img_size;
 
+        // 画像の最大サイズ(Mピクセル)
+        int max_img_pixel;
+
         // JD ホームページのアドレス
         std::string url_jdhp;
 
@@ -119,6 +152,24 @@ namespace CONFIG
         // bbsmenu.htmlの内にあるリンクは全て板とみなす
         bool use_link_as_board;
 
+        // 板移転時に確認ダイアログを表示する
+        bool show_movediag;
+
+        // スレタイ検索用メニュータイトルアドレス
+        std::string menu_search_title;
+        std::string url_search_title;
+
+        // スレタイ検索用正規表現
+        std::string regex_search_title;
+
+        // web検索用メニュータイトルアドレス
+        std::string menu_search_web;
+        std::string url_search_web;
+
+        // p2 書き込み用アドレス
+        std::string url_writep2;
+        std::string url_resp2;
+
         // 色
         std::vector< std::string > str_color;
 
@@ -128,33 +179,72 @@ namespace CONFIG
         // ツリービューの行間スペース
         int tree_ypad;
 
-        // boardビューで古いスレも表示
+        // 各ビューと枠との間の余白
+        int view_margin;
+
+        // スレ一覧で古いスレも表示
         bool show_oldarticle;
 
         // スレ一覧で指定した値(時間)よりも後に立てられたスレを新着とみなす
         int newthread_hour;
 
-        // ツリービューのスクロール量(行数)
+        // スレ一覧でインクリメント検索をする
+        bool inc_search_board;
+
+        // スレ一覧でdeleteを押したときに確認ダイアログを表示する
+        bool show_deldiag;
+
+        // ツリービューのスクロール量(マウスホイール上下・行数)
         int tree_scroll_size;
 
-        // スレビューのスクロール量
+        // スレビューのスクロール量(マウスホイール上下・行数)
         int scroll_size;
 
-        // スレビューのスクロール量(キー上下)
+        // スレビューのスクロール量(キー上下・行数)
         int key_scroll_size;
 
+        // スレビューの高速スクロール量(キー上下・ ページ高 - 行高 * key_fastscroll_size )
+        int key_fastscroll_size;
+
+        // スレビューでリロード後に一番下までスクロール
+        bool jump_after_reload;
+
+        // スレビューでリロード後に新着までスクロール
+        bool jump_new_after_reload;
+
+        // 実況モード
+        int live_mode;
+
+        // 実況速度
+        int live_speed;
+
+        // 実況のスクロールモードを切り替えるしきい値
+        int live_threshold;
+
         // 板一覧でカテゴリを常にひとつだけ開く
         bool open_one_category;
 
-        // 書き込み時に書き込み確認ダイアログを出すかどうか
+        // お気に入りでカテゴリを常にひとつだけ開く
+        bool open_one_favorite;
+
+        // 書き込み時に書き込み確認ダイアログを出さない
         bool always_write_ok;
 
         // 書き込みログを保存
         bool save_postlog;
 
+        // 書き込みログの最大サイズ
+        int maxsize_postlog;
+
+        // 書き込み履歴を保存
+        bool save_posthist;
+
         // 「書き込み中」のダイアログを表示しない
         bool hide_writing_dialog;
 
+        // 非アクティブ時に書き込みビューを折りたたむ
+        bool fold_message;
+
         // ポップアップとカーソルの間のマージン
         int margin_popup;
 
@@ -181,8 +271,10 @@ namespace CONFIG
         bool instruct_tglimg;
         bool instruct_tglimg_end;
 
-        // スレ表示の行間調整
+        // 下線位置
         double adjust_underline_pos;
+
+        // 行間スペース
         double adjust_line_space;
 
         // リンク下線を表示
@@ -191,15 +283,23 @@ namespace CONFIG
         // スレビューで文字幅の近似を厳密にする        
         bool strict_char_width;
 
+        // スレビューで発言数(ID)をカウントする
+        bool check_id;
+
+        // レス参照で色を変える回数
+        int num_reference_high;
+        int num_reference_low;
+
+        // 発言数で色を変える回数
+        int num_id_high;
+        int num_id_low;
+
         // datのパース時にURL判定を甘くする(^なども含める)
         bool loose_url;
 
         // ユーザーコマンドで選択できない項目を非表示にする
         bool hide_usrcmd;
 
-        // 指定した数よりもユーザーコマンドが多い場合はサブメニュー化する
-        int max_show_usrcmd;
-
         // スレビューで再読み込みボタンを押したときに全タブを更新する
         bool reload_allthreads;
 
@@ -209,12 +309,24 @@ namespace CONFIG
         // タブにアイコンを表示するか
         bool show_tab_icon;
 
+        // スレビューに書き込みマークを表示するか
+        bool show_post_mark;
+
+        // ボタンをフラットにするか
+        bool flat_button;
+
         // スレ あぼーん word
         std::list< std::string > list_abone_word_thread;
 
         // スレ あぼーん regex
         std::list< std::string > list_abone_regex_thread;
 
+        // スレ あぼーん レス数
+        int abone_number_thread;
+
+        // スレ あぼーん スレ立てからの経過時間
+        int abone_hour_thread;
+
         // あぼーん name
         std::list< std::string > list_abone_name;
 
@@ -231,6 +343,18 @@ namespace CONFIG
         // 右ペーンが空の時にサイドバーを閉じるか
         bool expand_sidebar;
 
+        // 3ペーン時にスレ一覧やスレビューを最大化するか
+        bool expand_rpane;
+
+        // 次スレ検索の類似度のしきい値
+        int threshold_next;
+
+        // 次スレを開いたときにお気に入りのアドレスと名前を自動更新
+        bool replace_favorite_next;
+
+        // スレをお気に入りに追加したときにしおりをセットする
+        bool bookmark_drop;
+
 #ifdef HAVE_MIGEMO_H
         // migemo-dictの場所
         std::string migemodict_path;
@@ -248,9 +372,23 @@ namespace CONFIG
         // 保存
         void save();
 
+        // フォントのリセット
+        void reset_fonts();
+
+        // 色のリセット
+        void reset_colors();
+
+        // プロクシ設定
+        void set_proxy_for2ch( const std::string& proxy );
+        void set_proxy_for2ch_w( const std::string& proxy );
+        void set_proxy_for_data( const std::string& proxy );
+
       private:
 
         void save_impl( const std::string& path );
+
+        void set_fonts( JDLIB::ConfLoader& cf );
+        void set_colors( JDLIB::ConfLoader& cf );
     };
     
 }
