@@ -29,15 +29,20 @@ namespace CONFIG
     
     // 前回開いたviewを復元するか
     const bool get_restore_board();
-    void set_restore_board( bool restore );
+    void set_restore_board( const bool restore );
     const bool get_restore_article();
-    void set_restore_article( bool restore );
+    void set_restore_article( const bool restore );
     const bool get_restore_image();
-    void set_restore_image( bool restore );
+    void set_restore_image( const bool restore );
+
+    // 自前でウィンドウ配置を管理する
+    const bool get_manage_winpos();
+    void set_manage_winpos( const bool manage );
 
     // 色 ( # + 12桁の16進数 の形式 )
     const std::string& get_color( int id );
     void set_color( int id, const std::string& color );
+    void reset_colors();
 
     // ツリービューでgtkrcの設定を使用するか
     const bool get_use_tree_gtkrc();
@@ -45,12 +50,19 @@ namespace CONFIG
     // ツリービューの行間スペース
     const int get_tree_ypad();
 
+    // 各ビューと枠との間の余白
+    const int get_view_margin();
+
     // フォント
     const std::string& get_fontname( int id );
     void set_fontname( int id, const std::string& fontname );
+    void reset_fonts();
 
     // レスを参照するときに前に付ける文字
-    const std::string& get_ref_prefix();
+    const std::string get_ref_prefix();
+
+    // 参照文字( ref_prefix ) の後のスペースの数
+    const int ref_prefix_space();
 
     // JD ホームページのアドレス
     const std::string& get_url_jdhp();
@@ -64,6 +76,10 @@ namespace CONFIG
     // bbsmenu.html内にあるリンクは全て板とみなす
     const bool use_link_as_board();
 
+    // 板移転時に確認ダイアログを表示する
+    const bool get_show_movediag();
+    void set_show_movediag( const bool show );
+
     // スレタイ検索用メニュータイトルアドレス
     const std::string& get_menu_search_title();
     const std::string& get_url_search_title();
@@ -75,29 +91,35 @@ namespace CONFIG
     const std::string& get_menu_search_web();
     const std::string& get_url_search_web();
 
+    // p2 書き込み用アドレス
+    const std::string& get_url_writep2();
+    const std::string& get_url_resp2();
+
     // 2ch にアクセスするときのエージェント名
     const std::string& get_agent_for2ch();
 
     // 2ch にログインするときのX-2ch-UA
     const std::string& get_x_2ch_ua();
 
-    // 読み込み用プロクシとポート番号
+    // 2ch 読み込み用プロクシとポート番号
     const bool get_use_proxy_for2ch();
     const std::string& get_proxy_for2ch();
     const int get_proxy_port_for2ch();
+    const std::string& get_proxy_basicauth_for2ch();
 
-    void set_use_proxy_for2ch( bool set );
+    void set_use_proxy_for2ch( const bool set );
     void set_proxy_for2ch( const std::string& proxy );
-    void set_proxy_port_for2ch( int port );
+    void set_proxy_port_for2ch( const int port );
 
-    // 書き込み用プロクシとポート番号
+    // 2ch 書き込み用プロクシとポート番号
     const bool get_use_proxy_for2ch_w();
     const std::string& get_proxy_for2ch_w();
     const int get_proxy_port_for2ch_w();
+    const std::string& get_proxy_basicauth_for2ch_w();
 
-    void set_use_proxy_for2ch_w( bool set );
+    void set_use_proxy_for2ch_w( const bool set );
     void set_proxy_for2ch_w( const std::string& proxy );
-    void set_proxy_port_for2ch_w( int port );
+    void set_proxy_port_for2ch_w( const int port );
 
     // 2ch外にアクセスするときのエージェント名
     const std::string& get_agent_for_data();
@@ -106,10 +128,11 @@ namespace CONFIG
     const bool get_use_proxy_for_data();
     const std::string& get_proxy_for_data();
     const int get_proxy_port_for_data();
+    const std::string& get_proxy_basicauth_for_data();
 
-    void set_use_proxy_for_data( bool set );
+    void set_use_proxy_for_data( const bool set );
     void set_proxy_for_data( const std::string& proxy );
-    void set_proxy_port_for_data( int port );
+    void set_proxy_port_for_data( const int port );
 
     // ローダのバッファサイズ
     const int get_loader_bufsize();
@@ -147,31 +170,38 @@ namespace CONFIG
 
     // 画像ポップアップを使用する
     const bool get_use_image_popup();
-    void set_use_image_popup( bool use );
+    void set_use_image_popup( const bool use );
 
     // 画像ビューを使用する
     const bool get_use_image_view();
-    void set_use_image_view( bool image_view );
+    void set_use_image_view( const bool image_view );
 
     // インライン画像表示をする
     const bool get_use_inline_image();
-    void set_use_inline_image( bool inline_img );
+    void set_use_inline_image( const bool inline_img );
+
+    // ssspアイコン表示
+    const bool get_show_ssspicon();
+    void set_show_sssp_icon( const bool show );
 
     // 画像にモザイクかける
     const bool get_use_mosaic();
-    void set_use_mosaic( bool mosaic );
+    void set_use_mosaic( const bool mosaic );
+
+    // モザイクの大きさ
+    const int get_mosaic_size();
 
     // 画像をデフォルトでウィンドウサイズに合わせる
     const bool get_zoom_to_fit();
-    void set_zoom_to_fit( bool fit );
+    void set_zoom_to_fit( const bool fit );
 
     // 画像キャッシュ削除の日数
     const int get_del_img_day();
-    void set_del_img_day( int day );
+    void set_del_img_day( const int day );
 
     // 画像あぼーん削除の日数
     const int get_del_imgabone_day();
-    void set_del_imgabone_day( int day );
+    void set_del_imgabone_day( const int day );
 
     // ダウンロードする画像の最大ファイルサイズ(Mbyte)
     const int get_max_img_size();
@@ -189,28 +219,62 @@ namespace CONFIG
     // スレ一覧でインクリメント検索をする
     const bool get_inc_search_board();
 
-    // ツリービューのスクロール量(行数)
+    // スレ一覧でdeleteを押したときに確認ダイアログを表示する
+    const bool get_show_deldiag();
+    void set_show_deldiag( const bool show );
+
+    // ツリービューのスクロール量(マウスホイール上下・行数)
     const int get_tree_scroll_size();
 
-    // スレビューのスクロール量
+    // スレビューのスクロール量(マウスホイール上下・行数)
     const int get_scroll_size();
 
-    // スレビューのスクロール量(キー上下)
+    // スレビューのスクロール量(キー上下・行数)
     const int get_key_scroll_size();
 
+    // スレビューの高速スクロール量(キー上下・ ページ高 - 行高 * key_fastscroll_size )
+    const int get_key_fastscroll_size();
+
     // スレビューでリロード後に一番下までスクロール
     const bool get_jump_after_reload();
+    void set_jump_after_reload( bool set );
+
+    // スレビューでリロード後に新着までスクロール
+    const bool get_jump_new_after_reload();
+    void set_jump_new_after_reload( bool set );
+
+    // 実況モード
+    const int get_live_mode();
+    void set_live_mode( const int mode );
+
+    // 実況速度
+    const int get_live_speed();
+    void set_live_speed( const int speed );
+
+    // 実況のスクロールモードを切り替えるしきい値
+    const int get_live_threshold();
+    void set_live_threshode( const int th );
 
     // 板一覧でカテゴリを常にひとつだけ開く
     const bool get_open_one_category();
 
+    // お気に入りでカテゴリを常にひとつだけ開く
+    const bool get_open_one_favorite();
+
     // 書き込み時に書き込み確認ダイアログを出すかどうか
     const bool get_always_write_ok();
     void set_always_write_ok( bool write_ok );
 
     // 書き込みログを保存
-    const bool get_save_postlog();
-    void set_save_postlog( bool save );
+    const bool get_save_post_log();
+    void set_save_post_log( bool save );
+
+    // 書き込みログの最大サイズ
+    const size_t get_maxsize_post_log();
+
+    // 書き込み履歴を保存
+    const bool get_save_post_history();
+    void set_save_post_history( bool save );
 
     // 書き込み中のダイアログを表示しない
     const bool get_hide_writing_dialog();
@@ -277,9 +341,7 @@ namespace CONFIG
 
     // ユーザーコマンドで選択できない項目を非表示にする
     const bool get_hide_usrcmd();
-
-    // 指定した数よりもユーザーコマンドが多い場合はサブメニュー化する
-    const int get_max_show_usrcmd();
+    void set_hide_usrcmd( bool hide );
 
     // スレビューで再読み込みボタンを押したときに全タブを更新する
     const bool get_reload_allthreads();
@@ -290,6 +352,14 @@ namespace CONFIG
     // タブにアイコンを表示するか
     const bool get_show_tab_icon();
 
+    // スレビューに書き込みマークを表示するか
+    const bool get_show_post_mark();
+    void set_show_post_mark( const bool show );
+
+    // ボタンをフラットにするか
+    const bool get_flat_button();
+    void set_flat_button( const bool set );
+
     // boardviewでのスレの全体あぼーん
     std::list< std::string >& get_list_abone_word_thread(); // ワード
     std::list< std::string >& get_list_abone_regex_thread(); // 正規表現
@@ -322,6 +392,18 @@ namespace CONFIG
     // 右ペーンが空の時にサイドバーを閉じるか
     const bool get_expand_sidebar();
 
+    // 3ペーン時にスレ一覧やスレビューを最大化するか
+    const bool get_expand_rpane();
+
+    // 次スレ検索の類似度のしきい値
+    const int get_threshold_next();
+
+    // 次スレを開いたときにお気に入りのアドレスと名前を自動更新
+    const bool get_replace_favorite_next();
+
+    // スレをお気に入りに追加したときにしおりをセットする
+    const bool get_bookmark_drop();
+
 #ifdef HAVE_MIGEMO_H
     // migemo-dictの場所
     const std::string& get_migemodict_path();
