@@ -104,14 +104,17 @@ namespace DBTREE
         int m_abone_number_thread; // レスの数
         int m_abone_hour_thread; // スレ立てからの経過時間
 
-        // ローカルプロキシ設定
+        // 読み込み用ローカルプロキシ設定
         int m_mode_local_proxy;
         std::string m_local_proxy;
         int m_local_proxy_port;
+        std::string m_local_proxy_basicauth; // basic 認証用の「ユーザID:パスワード」の組
 
+        // 書き込み用ローカルプロキシ設定
         int m_mode_local_proxy_w;
         std::string m_local_proxy_w;
         int m_local_proxy_port_w;
+        std::string m_local_proxy_basicauth_w; // basic 認証用の「ユーザID:パスワード」の組
 
         // 書き込み時のデフォルトの名前とメアド
         std::string m_write_name;      
@@ -132,6 +135,8 @@ namespace DBTREE
         // 最大レス数
         int m_number_max_res;
 
+        // ダウンロード用変数
+        std::list< std::string > m_url_update_views; // CORE::core_set_command( "update_board" ) を送信するビューのアドレス
         char* m_rawdata;
         int m_lng_rawdata;
 
@@ -147,8 +152,9 @@ namespace DBTREE
         // クッキー, 書き込み時に必要
         std::list< std::string > m_list_cookies_for_write;
 
-        // hana, 2ch書き込み時に必要
-        std::string m_hana_for_write;   
+        // 書き込み時に必要なキーワード( hana=mogera や suka=pontan など )
+        // 書き込み時のメッセージに付加する
+        std::string m_keyword_for_write;   
 
         // basic 認証用の「ユーザID:パスワード」の組
         std::string m_basicauth;
@@ -225,12 +231,18 @@ namespace DBTREE
         void update_name( const std::string& name );
         const std::string& get_subjecttxt() const { return m_subjecttxt; }
 
-        // ダウンロード、書き込み時のエージェント名やプロキシ
+        // ダウンロード、書き込み時のエージェント名
         virtual const std::string& get_agent();
+
+        // ダウンロード時のプロキシ
         virtual const std::string get_proxy_host();
         virtual const int get_proxy_port();
+        virtual const std::string get_proxy_basicauth();
+
+        // 書き込み時のプロキシ
         virtual const std::string get_proxy_host_w();
         virtual const int get_proxy_port_w();
+        virtual const std::string get_proxy_basicauth_w();
 
         // ローカルルール
         virtual const std::string localrule();
@@ -252,9 +264,14 @@ namespace DBTREE
         const std::list< std::string >& list_cookies_for_write() { return m_list_cookies_for_write; }
         void set_list_cookies_for_write( const std::list< std::string >& list_cookies ){ m_list_cookies_for_write = list_cookies; }
 
-        // hana
-        const std::string& hana_for_write() { return m_hana_for_write; }
-        void set_hana_for_write( const std::string& hana ){ m_hana_for_write = hana; }
+        // 書き込み時に必要なキーワード( hana=mogera や suka=pontan など )
+        // 書き込み時のメッセージに付加する
+        const std::string& get_keyword_for_write() const { return m_keyword_for_write; }
+        void set_keyword_for_write( const std::string& keyword ){ m_keyword_for_write = keyword; }
+
+        // 書き込み時に必要なキーワード( hana=mogera や suka=pontan など )を
+        // 確認画面のhtmlから解析する      
+        virtual void analyze_keyword_for_write( const std::string& html ){}
 
         // basic認証
         const std::string& get_basicauth() const { return m_basicauth; }
@@ -314,7 +331,9 @@ namespace DBTREE
         ArticleBase* get_article_create( const std::string& id );
         ArticleBase* get_article_fromURL( const std::string& url );
 
-        void download_subject();
+        // subject.txt ダウンロード
+        // url_update_view : CORE::core_set_command( "update_board" ) を送信するビューのアドレス
+        void download_subject( const std::string& url_update_view );
 
         // 新スレ作成用のメッセージ変換
         virtual const std::string create_newarticle_message( const std::string& subject,
@@ -365,15 +384,19 @@ namespace DBTREE
         const int get_mode_local_proxy() const { return m_mode_local_proxy; }
         const std::string& get_local_proxy() const { return m_local_proxy; }
         const int get_local_proxy_port() const { return m_local_proxy_port; }
+        const std::string& get_local_proxy_basicauth() const { return m_local_proxy_basicauth; }
+
         void set_mode_local_proxy( int mode ){ m_mode_local_proxy = mode; }
-        void set_local_proxy( const std::string& proxy ){ m_local_proxy = proxy; }
+        void set_local_proxy( const std::string& proxy );
         void set_local_proxy_port( int port ){ m_local_proxy_port = port; }
 
         const int get_mode_local_proxy_w() const { return m_mode_local_proxy_w; }
         const std::string& get_local_proxy_w() const { return m_local_proxy_w; }
         const int get_local_proxy_port_w() const { return m_local_proxy_port_w; }
+        const std::string& get_local_proxy_basicauth_w() const { return m_local_proxy_basicauth_w; }
+
         void set_mode_local_proxy_w( int mode ){ m_mode_local_proxy_w = mode; }
-        void set_local_proxy_w( const std::string& proxy ){ m_local_proxy_w = proxy; }
+        void set_local_proxy_w( const std::string& proxy );
         void set_local_proxy_port_w( int port ){ m_local_proxy_port_w = port; }
 
         // 書き込み時のデフォルトの名前とメアド
@@ -391,6 +414,9 @@ namespace DBTREE
         void set_samba_sec( time_t sec ){ m_samba_sec = sec; }
         time_t get_write_leftsec(); // 書き込み可能までの残り秒
 
+        // 全書き込み履歴クリア
+        void clear_all_post_history();
+
         // 実況の秒数
         const time_t get_live_sec() const{ return m_live_sec; }
         void set_live_sec( time_t sec ){ m_live_sec = sec; }
@@ -415,6 +441,9 @@ namespace DBTREE
 
         void clear();
 
+        // m_url_update_views に登録されている view に update_board コマンドを送る
+        void send_update_board();
+
         // キャッシュのファイル名が正しいかどうか
         virtual bool is_valid( const std::string& filename ){ return false; }
 
