@@ -15,19 +15,24 @@
 #include "dbimg/img.h"
 
 #include "jdlib/miscutil.h"
+#include "jdlib/miscgtk.h"
+
+#include "control/controlutil.h"
+#include "control/controlid.h"
 
 #include "cache.h"
 #include "command.h"
 #include "sharedbuffer.h"
 #include "global.h"
-#include "controlutil.h"
-#include "controlid.h"
+#include "type.h"
 #include "prefdiagfactory.h"
+#include "httpcode.h"
+#include "session.h"
 
 #include <sstream>
 
 
-#define SIZE_MENU { 25, 50, 75, 100, 150, 200, 300, 400 }
+#define SIZE_MENU { 25, 50, 75, 100, 150, 200, 400 }
 
 
 
@@ -37,6 +42,7 @@ using namespace IMAGE;
 ImageViewBase::ImageViewBase( const std::string& url, const std::string& arg1, const std::string& arg2 )
     : SKELETON::View( url ),
       m_loading( false ),
+      m_under_mouse( false ),
       m_enable_menuslot( true )
 {
     // 高速化のためデータベースに直接アクセス
@@ -45,9 +51,6 @@ ImageViewBase::ImageViewBase( const std::string& url, const std::string& arg1, c
 
     // マウスジェスチャ可能
     set_enable_mg( true );
-
-    // コントロールモード設定
-    get_control().add_mode( CONTROL::MODE_IMAGE );
 }
 
 
@@ -60,6 +63,12 @@ ImageViewBase::~ImageViewBase()
 }
 
 
+SKELETON::Admin* ImageViewBase::get_admin()
+{
+    return IMAGE::get_admin();
+}
+
+
 //
 // 共通セットアップ
 //
@@ -80,6 +89,8 @@ void ImageViewBase::setup_common()
     m_event.signal_motion_notify_event().connect(  sigc::mem_fun( *this, &ImageViewBase::slot_motion_notify ) );
     m_event.signal_key_press_event().connect( sigc::mem_fun(*this, &ImageViewBase::slot_key_press ) );
     m_event.signal_scroll_event().connect( sigc::mem_fun(*this, &ImageViewBase::slot_scroll_event ) );
+    m_event.signal_enter_notify_event().connect(  sigc::mem_fun( *this, &ImageViewBase::slot_enter_notify_event ) );
+    m_event.signal_leave_notify_event().connect(  sigc::mem_fun( *this, &ImageViewBase::slot_leave_notify_event ) );
 
     m_event.grab_focus();
 
@@ -88,8 +99,10 @@ void ImageViewBase::setup_common()
     action_group() = Gtk::ActionGroup::create();
     action_group()->add( Gtk::Action::create( "CancelMosaic", "CancelMosaic"),
                          sigc::mem_fun( *this, &ImageViewBase::slot_cancel_mosaic ) );
-    action_group()->add( Gtk::Action::create( "LoadStop", "ロード中止"), sigc::mem_fun( *this, &ImageViewBase::stop ) );
-    action_group()->add( Gtk::Action::create( "Reload", "強制再読み込み"), sigc::mem_fun( *this, &ImageViewBase::slot_reload_force ) );
+    action_group()->add( Gtk::Action::create( "ShowLargeImg", "サイズが大きい画像を表示(_G)"),
+                         sigc::mem_fun( *this, &ImageViewBase::slot_show_large_img ) );
+    action_group()->add( Gtk::Action::create( "LoadStop", "StopLoading" ), sigc::mem_fun( *this, &ImageViewBase::stop ) );
+    action_group()->add( Gtk::Action::create( "Reload", "強制再読み込み(_E)"), sigc::mem_fun( *this, &ImageViewBase::slot_reload_force ) );
     action_group()->add( Gtk::Action::create( "AppendFavorite", "AppendFavorite"), sigc::mem_fun( *this, &ImageViewBase::slot_favorite ) );
 
     action_group()->add( Gtk::Action::create( "ZoomFitImage", "ZoomFitImage" ),
@@ -101,44 +114,49 @@ void ImageViewBase::setup_common()
     action_group()->add( Gtk::Action::create( "OrgSizeImage", "OrgSizeImage" ),
                          sigc::bind< int >( sigc::mem_fun( *this, &ImageViewBase::slot_resize_image ), 100 ) );
 
-    action_group()->add( Gtk::Action::create( "Size_Menu", "サイズ変更" ) );
+    action_group()->add( Gtk::Action::create( "Size_Menu", "サイズ変更(_R)" ) );
 
     // サイズ
     unsigned int size[] = SIZE_MENU;
     for( unsigned int i = 0; i < sizeof( size )/sizeof( unsigned int ) ; ++i ){
         int tmp_size = size[ i ];
         std::string str_size = MISC::itostr( tmp_size );
-        Glib::RefPtr< Gtk::Action > action = Gtk::Action::create( "Size" + str_size, str_size + "%" );
+        //ショートカットは、1から始まる
+        std::string str_shortcut = "(_" + MISC::itostr( i+1 ) + ")";
+        Glib::RefPtr< Gtk::Action > action = Gtk::Action::create( "Size" + str_size, str_size + "%" + str_shortcut );
         action_group()->add( action, sigc::bind< int >( sigc::mem_fun( *this, &ImageViewBase::slot_resize_image ), tmp_size ) );
     }
 
-    action_group()->add( Gtk::Action::create( "Move_Menu", "移動" ) );
-    action_group()->add( Gtk::Action::create( "MoveHead", "先頭に移動" ), sigc::mem_fun( *this, &ImageViewBase::slot_move_head ) );
-    action_group()->add( Gtk::Action::create( "MoveTail", "最後に移動" ), sigc::mem_fun( *this, &ImageViewBase::slot_move_tail ) );
+    action_group()->add( Gtk::Action::create( "Move_Menu", "移動(_M)" ) );
+    action_group()->add( Gtk::Action::create( "MoveHead", "先頭に移動(_H)" ), sigc::mem_fun( *this, &ImageViewBase::slot_move_head ) );
+    action_group()->add( Gtk::Action::create( "MoveTail", "最後に移動(_T)" ), sigc::mem_fun( *this, &ImageViewBase::slot_move_tail ) );
 
     action_group()->add( Gtk::Action::create( "Quit", "Quit" ), sigc::mem_fun( *this, &ImageViewBase::close_view ) );
 
-    action_group()->add( Gtk::Action::create( "Close_Menu", "複数の画像を閉じる" ) );
-    action_group()->add( Gtk::Action::create( "CloseOther", "他の画像" ), sigc::mem_fun( *this, &ImageViewBase::slot_close_other_views ) );
-    action_group()->add( Gtk::Action::create( "CloseLeft", "左←の画像" ), sigc::mem_fun( *this, &ImageViewBase::slot_close_left_views ) );
-    action_group()->add( Gtk::Action::create( "CloseRight", "右→の画像" ), sigc::mem_fun( *this, &ImageViewBase::slot_close_right_views ) );
-    action_group()->add( Gtk::Action::create( "CloseAll", "全ての画像" ), sigc::mem_fun( *this, &ImageViewBase::slot_close_all_views ) );
+    action_group()->add( Gtk::Action::create( "Close_Menu", "複数の画像を閉じる(_L)" ) );
+    action_group()->add( Gtk::Action::create( "CloseOther", "他の画像(_O)" ), sigc::mem_fun( *this, &ImageViewBase::slot_close_other_views ) );
+    action_group()->add( Gtk::Action::create( "CloseLeft", "左←の画像(_L)" ), sigc::mem_fun( *this, &ImageViewBase::slot_close_left_views ) );
+    action_group()->add( Gtk::Action::create( "CloseRight", "右→の画像(_R)" ), sigc::mem_fun( *this, &ImageViewBase::slot_close_right_views ) );
+    action_group()->add( Gtk::Action::create( "CloseAll", "全ての画像(_A)" ), sigc::mem_fun( *this, &ImageViewBase::slot_close_all_views ) );
 
-    action_group()->add( Gtk::Action::create( "OpenBrowser", "ブラウザで開く"),
+    action_group()->add( Gtk::ToggleAction::create( "LockTab", "タブをロックする(_K)", std::string(), false ),
+                         sigc::mem_fun( *this, &ImageViewBase::slot_lock ) );
+
+    action_group()->add( Gtk::Action::create( "OpenBrowser", "ブラウザで開く(_W)"),
                          sigc::mem_fun( *this, &ImageViewBase::slot_open_browser ) );
-    action_group()->add( Gtk::Action::create( "OpenRef", "参照元のレスを開く"), sigc::mem_fun( *this, &ImageViewBase::slot_open_ref ) );
-    action_group()->add( Gtk::Action::create( "CopyURL", "URLをコピー"), sigc::mem_fun( *this, &ImageViewBase::slot_copy_url ) );
+    action_group()->add( Gtk::Action::create( "OpenRef", "参照元のレスを開く(_O)"), sigc::mem_fun( *this, &ImageViewBase::slot_open_ref ) );
+    action_group()->add( Gtk::Action::create( "CopyURL", "URLをコピー(_U)"), sigc::mem_fun( *this, &ImageViewBase::slot_copy_url ) );
     action_group()->add( Gtk::Action::create( "Save", "Save"), sigc::mem_fun( *this, &ImageViewBase::slot_save ) );
-    action_group()->add( Gtk::Action::create( "SaveAll", "全ての画像を保存"), sigc::mem_fun( *this, &ImageViewBase::slot_save_all ) );
+    action_group()->add( Gtk::Action::create( "SaveAll", "全ての画像を保存(_A)..."), sigc::mem_fun( *this, &ImageViewBase::slot_save_all ) );
 
     action_group()->add( Gtk::Action::create( "DeleteMenu", "Delete" ) );    
-    action_group()->add( Gtk::Action::create( "DeleteImage", "削除する"), sigc::mem_fun( *this, &ImageViewBase::delete_view ) );
-    action_group()->add( Gtk::ToggleAction::create( "ProtectImage", "キャッシュを保護する", std::string(), false ),
+    action_group()->add( Gtk::Action::create( "DeleteImage", "削除する(_D)"), sigc::mem_fun( *this, &ImageViewBase::delete_view ) );
+    action_group()->add( Gtk::ToggleAction::create( "ProtectImage", "キャッシュを保護する(_H)", std::string(), false ),
                          sigc::mem_fun( *this, &ImageViewBase::slot_toggle_protectimage ) );
 
-    action_group()->add( Gtk::Action::create( "AboneImage", "画像をあぼ〜んする"), sigc::mem_fun( *this, &ImageViewBase::slot_abone_img ) );
+    action_group()->add( Gtk::Action::create( "AboneImage", "画像をあぼ〜んする(_B)"), sigc::mem_fun( *this, &ImageViewBase::slot_abone_img ) );
 
-    action_group()->add( Gtk::Action::create( "Preference", "Property"), sigc::mem_fun( *this, &ImageViewBase::slot_preference ) );
+    action_group()->add( Gtk::Action::create( "Preference", "プロパティ(_P)..."), sigc::mem_fun( *this, &ImageViewBase::show_preference ) );
 
     ui_manager() = Gtk::UIManager::create();    
     ui_manager()->insert_action_group( action_group() );
@@ -150,6 +168,10 @@ void ImageViewBase::setup_common()
     "<popup name='popup_menu'>"
 
     "<menuitem action='CancelMosaic'/>"
+    "<menuitem action='ShowLargeImg'/>"
+    "<separator/>"
+
+    "<menuitem action='Quit'/>"
     "<separator/>"
 
     "<menu action='Size_Menu'>"
@@ -209,6 +231,9 @@ void ImageViewBase::setup_common()
     "</menu>"
     "<separator/>"
 
+    "<menuitem action='LockTab'/>"
+    "<separator/>"
+
     "<menuitem action='Quit'/>"
     "<separator/>"
 
@@ -257,7 +282,7 @@ void ImageViewBase::setup_common()
     ui_manager()->add_ui_from_string( str_ui );
 
 
-    // ポップアップメニューにキーアクセレータを表示
+    // ポップアップメニューにキーアクセレータやマウスジェスチャを表示
     Gtk::Menu* popupmenu = dynamic_cast< Gtk::Menu* >( ui_manager()->get_widget( "/popup_menu" ) );
     CONTROL::set_menu_motion( popupmenu );
 
@@ -299,7 +324,7 @@ void ImageViewBase::remove_imagearea()
 //
 // コマンド
 //
-bool ImageViewBase::set_command( const std::string& command, const std::string& arg )
+const bool ImageViewBase::set_command( const std::string& command, const std::string& arg1, const std::string& arg2 )
 {
     if( command == "switch_icon" ) switch_icon();
 
@@ -318,7 +343,8 @@ void ImageViewBase::reload()
 #endif
 
     std::string refurl = m_img->get_refurl();
-    m_img->download_img( refurl );
+    const bool nomosaic = false;
+    m_img->download_img( refurl, nomosaic );
 }
 
 
@@ -421,7 +447,7 @@ void ImageViewBase::slot_close_all_views()
 //
 // プロパティ
 //
-void ImageViewBase::slot_preference()
+void ImageViewBase::show_preference()
 {
     SKELETON::PrefDiag* pref= CORE::PrefDiagFactory( IMAGE::get_admin()->get_win(), CORE::PREFDIAG_IMAGE, get_url() );
 
@@ -446,7 +472,7 @@ void ImageViewBase::delete_view()
 //
 // viewの操作
 //
-void ImageViewBase::operate_view( const int& control )
+const bool ImageViewBase::operate_view( const int control )
 {
 #ifdef _DEBUG
     std::cout << "ImageViewBase::operate_view control = " << control << std::endl;
@@ -455,6 +481,7 @@ void ImageViewBase::operate_view( const int& control )
     switch( control ){
 
         case CONTROL::CancelMosaic:
+        case CONTROL::CancelMosaicButton:
             slot_cancel_mosaic();
             break;
 
@@ -483,7 +510,8 @@ void ImageViewBase::operate_view( const int& control )
             stop();
             break;
 
-        case CONTROL::CloseTabButton:
+        case CONTROL::CloseImageButton:
+        case CONTROL::CloseImageTabButton:
         case CONTROL::Quit:
             close_view();
             break;
@@ -496,19 +524,19 @@ void ImageViewBase::operate_view( const int& control )
             IMAGE::get_admin()->set_command( "tab_right" );
             break;
 
-        case CONTROL::ScrollUp:
+        case CONTROL::ScrollUpImage:
             IMAGE::get_admin()->set_command( "scroll_up" );
             break;
 
-        case CONTROL::ScrollDown:
+        case CONTROL::ScrollDownImage:
             IMAGE::get_admin()->set_command( "scroll_down" );
             break;
 
-        case CONTROL::ScrollLeft:
+        case CONTROL::ScrollLeftImage:
             IMAGE::get_admin()->set_command( "scroll_left" );
             break;
 
-        case CONTROL::ScrollRight:
+        case CONTROL::ScrollRightImage:
             IMAGE::get_admin()->set_command( "scroll_right" );
             break;
 
@@ -530,9 +558,9 @@ void ImageViewBase::operate_view( const int& control )
             if( !m_img->is_protected() ){
 
                 SKELETON::MsgDiag mdiag( IMAGE::get_admin()->get_win(),
-                                         "画像を削除しますか？", false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_OK_CANCEL );
-                mdiag.set_default_response( Gtk::RESPONSE_OK );
-                if( mdiag.run() == Gtk::RESPONSE_OK ) delete_view();
+                                         "画像を削除しますか？", false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO );
+                mdiag.set_default_response( Gtk::RESPONSE_NO );
+                if( mdiag.run() == Gtk::RESPONSE_YES ) delete_view();
             }
             else{
 
@@ -551,7 +579,17 @@ void ImageViewBase::operate_view( const int& control )
         case CONTROL::ShowMenuBar:
             CORE::core_set_command( "toggle_menubar" );
             break;
+
+            // お気に入りに追加
+        case CONTROL::AppendFavorite:
+            slot_favorite();
+            break;
+
+        default:
+            return false;
     }
+
+    return true;
 }
                                                     
 
@@ -567,9 +605,7 @@ bool ImageViewBase::slot_key_press( GdkEventKey* event )
     std::cout << "ImageViewBase::slot_key_press url = " << get_url() << std::endl;
 #endif
 
-    operate_view( get_control().key_press( event ) );
-
-    return true;
+    return operate_view( get_control().key_press( event ) );
 }
 
 
@@ -590,6 +626,7 @@ bool ImageViewBase::slot_button_press( GdkEventButton* event )
     get_control().MG_wheel_start( event );
 
     // ダブルクリック
+    // button_release_eventでは event->type に必ず GDK_BUTTON_RELEASE が入る
     m_dblclick = false;
     if( event->type == GDK_2BUTTON_PRESS ) m_dblclick = true; 
 
@@ -632,7 +669,7 @@ bool ImageViewBase::slot_button_release( GdkEventButton* event )
         show_popupmenu( "", false );
     }
 
-    else operate_view( get_control().button_press( event ) );
+    else if( is_under_mouse() ) operate_view( get_control().button_press( event ) );
 
     event->type = type_copy;
 
@@ -670,6 +707,31 @@ bool ImageViewBase::slot_scroll_event( GdkEventScroll* event )
 }
 
 
+//
+// マウスが入った
+//
+bool ImageViewBase::slot_enter_notify_event( GdkEventCrossing* event )
+{
+#ifdef _DEBUG
+    std::cout << "ImageViewBase::slot_enter_notify_event\n";
+#endif
+    m_under_mouse = true;
+
+    return true;
+}
+
+//
+// マウスが出た
+//
+bool ImageViewBase::slot_leave_notify_event( GdkEventCrossing* event )
+{
+#ifdef _DEBUG
+    std::cout << "ImageViewBase::slot_leave_notify_event\n";
+#endif
+    m_under_mouse = false;
+
+    return true;
+}
 
 
 //
@@ -685,7 +747,10 @@ void ImageViewBase::slot_reload_force()
         return;
     }
 
-    m_img->set_code( 0 );
+    // コードをリセットしてから再読み込みをかける
+    m_img->reset();
+    m_img->set_code( HTTP_INIT );
+    m_img->set_type( DBIMG::T_UNKNOWN );
     reload();
     CORE::core_set_command( "redraw", get_url() );
 }
@@ -699,17 +764,46 @@ void ImageViewBase::slot_cancel_mosaic()
     if( ! m_enable_menuslot ) return;
 
     if( ! m_img->is_cached() ) return;
+
+    if( m_img->is_fake() ){
+
+        std::string type = "本当の画像タイプは";
+        switch( DBIMG::get_type_real( get_url() ) ){
+            case DBIMG::T_JPG: type += "JPEG"; break;
+            case DBIMG::T_PNG: type += "PNG"; break;
+            case DBIMG::T_GIF: type += "GIF"; break;
+            case DBIMG::T_BMP: type += "BMP"; break;
+        }
+        type += "です。";
+
+        SKELETON::MsgDiag mdiag( IMAGE::get_admin()->get_win(),
+                                 "拡張子が偽装されています。" + type + "\n\nモザイクを解除しますか？",
+                                 false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO );
+
+        mdiag.set_default_response( Gtk::RESPONSE_NO );
+        if( mdiag.run() != Gtk::RESPONSE_YES ) return;
+    }
+
     m_img->set_mosaic( false );
     CORE::core_set_command( "redraw", get_url() );
 }
 
 
+//
+// 大きい画像を表示
+//
+void ImageViewBase::slot_show_large_img()
+{
+    m_img->show_large_img();
+}
+
 
 //
 // ウィンドウにサイズを合わせる
 //
 void ImageViewBase::slot_fit_win()
 {
+    if( m_img->is_zoom_to_fit() ) SESSION::toggle_img_fit_mode();
     m_img->set_zoom_to_fit( true );
     CORE::core_set_command( "redraw", get_url() );
 }
@@ -794,6 +888,18 @@ void ImageViewBase::slot_resize_image( int size )
 }
 
 
+//
+// ロックする
+//
+void ImageViewBase::slot_lock()
+{
+    if( ! m_enable_menuslot ) return;
+
+    if( is_locked() ) unlock();
+    else lock();
+}
+
+
 //
 // ブラウザで開く
 //
@@ -815,10 +921,11 @@ void ImageViewBase::slot_open_ref()
 {
     if( ! m_enable_menuslot ) return;
 
-    std::string refurl = m_img->get_refurl();
+    const std::string refurl = m_img->get_refurl();
 
     int center, from, to;
-    std::string url = DBTREE::url_dat( refurl, center, to );
+    std::string num_str;
+    const std::string url = DBTREE::url_dat( refurl, center, to, num_str );
     if( url.empty() ) return;
 
     const int range = 10;
@@ -839,7 +946,7 @@ void ImageViewBase::slot_copy_url()
 {
     if( ! m_enable_menuslot ) return;
 
-    COPYCLIP( get_url() );
+    MISC::CopyClipboard( get_url() );
 }
 
 
@@ -910,9 +1017,12 @@ void ImageViewBase::set_image_to_buffer()
     info.type = TYPE_IMAGE;
     info.url = get_url();
     info.name = get_url();
+    info.data = std::string();
+    info.path = Gtk::TreePath( "0" ).to_string();
 
-    CORE::SBUF_clear_info();
-    CORE::SBUF_append( info );
+    CORE::DATA_INFO_LIST list_info;
+    list_info.push_back( info );
+    CORE::SBUF_set_list( list_info );
 }
 
 
@@ -934,6 +1044,22 @@ void ImageViewBase::activate_act_before_popupmenu( const std::string& url )
 
     bool current_protect = m_img->is_protected();
 
+    // ロック
+    act = action_group()->get_action( "LockTab" );
+    if( act ){
+
+        Glib::RefPtr< Gtk::ToggleAction > tact = Glib::RefPtr< Gtk::ToggleAction >::cast_dynamic( act ); 
+        if( is_locked() ) tact->set_active( true );
+        else tact->set_active( false );
+    }
+
+    // 閉じる
+    act = action_group()->get_action( "Quit" );
+    if( act ){
+        if( is_locked() ) act->set_sensitive( false );
+        else act->set_sensitive( true );
+    }
+
     // モザイク
     act = action_group()->get_action( "CancelMosaic" );
     if( act ){
@@ -941,11 +1067,26 @@ void ImageViewBase::activate_act_before_popupmenu( const std::string& url )
         else act->set_sensitive( false );
     }
 
+    // サイズの大きい画像を表示
+    act = action_group()->get_action( "ShowLargeImg" );
+    if( act ){
+        if( m_img->get_type() == DBIMG::T_LARGE ) act->set_sensitive( true );
+        else act->set_sensitive( false );
+    }
+
     // サイズ系メニュー、お気に入り、保存
-    char* sizemenu[] = { "Size_Menu", "OrgSizeImage", "ZoomFitImage", "ZoomInImage", "ZoomOutImage",
-                         "AppendFavorite", "Save" };
+    std::string sizemenus[] =
+    {
+        "Size_Menu",
+        "OrgSizeImage",
+        "ZoomFitImage",
+        "ZoomInImage",
+        "ZoomOutImage",
+        "AppendFavorite",
+        "Save"
+    };
     for( int i = 0; i < 7; ++i ){
-        act = action_group()->get_action( sizemenu[ i ] );
+	    act = action_group()->get_action( sizemenus[ i ] );
         if( act ){
             if( m_img->is_cached() ) act->set_sensitive( true );
             else act->set_sensitive( false );
@@ -979,7 +1120,7 @@ void ImageViewBase::activate_act_before_popupmenu( const std::string& url )
     // 削除
     act = action_group()->get_action( "DeleteMenu" );
     if( act ){
-        if( m_img->is_cached() && ! m_img->is_protected() ) act->set_sensitive( true );
+        if(  m_img->get_code() != HTTP_INIT && ! m_img->is_protected() ) act->set_sensitive( true );
         else act->set_sensitive( false );
     }
 
@@ -993,7 +1134,7 @@ void ImageViewBase::activate_act_before_popupmenu( const std::string& url )
     // あぼーん
     act = action_group()->get_action( "AboneImage" );
     if( act ){
-        if( m_img->is_cached() && ! m_img->is_protected() ) act->set_sensitive( true );
+        if( ! m_img->is_protected() ) act->set_sensitive( true );
         else act->set_sensitive( false );
     }
 
