@@ -7,9 +7,33 @@
 
 #include <string>
 #include <list>
+#include <vector>
+#include <glibmm.h>
 
 namespace MISC
 {
+    // URLスキームタイプ
+	enum
+	{
+		SCHEME_NONE,
+
+		SCHEME_HTTP,
+		SCHEME_FTP,
+		SCHEME_TTP,
+		SCHEME_TP,
+                SCHEME_SSSP
+	};
+
+     // get_ucs2mode()の戻り値
+     enum
+     {
+         UCS2MODE_BASIC_LATIN = 0,
+         UCS2MODE_HIRA,
+         UCS2MODE_KATA,
+
+         UCS2MODE_OTHER
+     };
+
     // str を "\n" ごとに区切ってlistにして出力
     std::list< std::string > get_lines( const std::string& str );
 
@@ -31,12 +55,12 @@ namespace MISC
     // list_inからコメント行(#)を除いてリストを返す
     std::list< std::string > remove_commentline_from_list( std::list< std::string >& list_in );
 
-    // 空白とカンマで区切られた str_in の文字列をリストにして出力
+    // 空白と""で区切られた str_in の文字列をリストにして出力
     // \"は " に置換される
     // (例)  "aaa" "bbb" "\"ccc\""  → aaa と bbb と "ccc"
     std::list< std::string > strtolist( std::string& str_in );
 
-    // list_in の文字列リストを空白とカンマで区切ってストリングにして出力
+    // list_in の文字列リストを空白と""で区切ってストリングにして出力
     // "は \" に置換される
     // (例)  "aaa" "bbb" "\"ccc\""
     std::string listtostr( std::list< std::string >& list_in );
@@ -65,6 +89,9 @@ namespace MISC
     // list_inから str1 を str2 に置き換えてリストを返す
     std::list< std::string > replace_str_list( std::list< std::string >& list_in, const std::string& str1, const std::string& str2 );
 
+    // str_in に含まれる改行文字を replace に置き換え
+    std::string replace_newlines_to_str( const std::string& str_in, const std::string& replace );
+
     // " を \" に置き換え
     std::string replace_quot( const std::string& str );
 
@@ -74,6 +101,9 @@ namespace MISC
     // str 中に含まれている str2 の 数を返す
     int count_str( const std::string& str, const std::string& str2 );
     
+    // str 中に含まれている chr の 数を返す
+    const int count_chr( const std::string& str, const char chr );
+
     //文字 -> 整数変換
     int str_to_uint( const char* str, unsigned int& dig, unsigned int& n );
 
@@ -83,15 +113,27 @@ namespace MISC
     // listで指定した数字を文字に変換
     std::string intlisttostr( std::list< int >& list_num );
 
+    // 16進数表記文字をバイナリに変換する
+    size_t chrtobin( const char* chr_in, char* chr_out );
+
     // strが半角でmaxsize文字を超えたらカットして後ろに...を付ける
     std::string cut_str( const std::string& str, unsigned int maxsize );
 
     // HTMLエスケープ
-    std::string html_escape( const std::string& str );
+    std::string html_escape( const std::string& str, const bool include_url = true );
 
     // HTMLアンエスケープ
     std::string html_unescape( const std::string& str );
 
+    // URL中のスキームを判別する
+    int is_url_scheme( const char* str_in, int* length = NULL );
+
+    // URLとして扱う文字かどうか判別する
+    bool is_url_char( const char* str_in, const bool loose_url );
+
+    // URLデコード
+    std::string url_decode( const std::string& url );
+
     // urlエンコード
     std::string url_encode( const char* str, size_t n );
 
@@ -99,6 +141,10 @@ namespace MISC
     // str は UTF-8 であること
     std::string charset_url_encode( const std::string& str, const std::string& charset );
 
+    // 文字コード変換して url エンコード
+    // ただし半角スペースのところを+に置き換えて区切る
+    std::string charset_url_encode_split( const std::string& str, const std::string& charset );
+
     // BASE64
     std::string base64( const std::string& str );
 
@@ -109,6 +155,9 @@ namespace MISC
     // utf-8 -> ucs2 変換
     int utf8toucs2( const char* utfstr, int& byte );
 
+    // ucs2 の種類
+    int get_ucs2mode( const int ucs2 );
+
     // ucs2 -> utf8 変換
     int ucs2toutf8( int ucs2, char* utfstr );
 
@@ -130,6 +179,9 @@ namespace MISC
 
     // path からファイル名を除いてディレクトリだけ取り出す
     std::string get_dir( const std::string& path );
+
+    // 文字数を限定して環境変数の値を返す
+    std::string getenv_limited( const char *name, const size_t size = 1 );
 }
 
 
