@@ -153,6 +153,9 @@ std::vector< std::string > delete_list;
 
 std::vector< std::string > live_urls;
 
+int img_fit_mode;
+
+std::string dir_select_favorite;
 
 /////////////////////////////////////
 
@@ -190,6 +193,7 @@ std::vector< int > parse_items( const std::string& items_str )
         if( *it == ITEM_NAME_RELOAD ) items.push_back( ITEM_RELOAD );
         if( *it == ITEM_NAME_STOPLOADING ) items.push_back( ITEM_STOPLOADING );
         if( *it == ITEM_NAME_FAVORITE ) items.push_back( ITEM_FAVORITE );
+        if( *it == ITEM_NAME_CHECK_UPDATE_ROOT ) items.push_back( ITEM_CHECK_UPDATE_ROOT );
         if( *it == ITEM_NAME_DELETE ) items.push_back( ITEM_DELETE );
         if( *it == ITEM_NAME_QUIT ) items.push_back( ITEM_QUIT );
         if( *it == ITEM_NAME_PREVVIEW ) items.push_back( ITEM_PREVVIEW );
@@ -216,6 +220,9 @@ std::vector< int > parse_items( const std::string& items_str )
 void read_list_urls( JDLIB::ConfLoader& cf, const std::string& id_urls, const std::string& id_locked,
                      std::list< std::string >& list_urls , std::list< bool >& list_locked )
 {
+    list_urls.clear();
+    list_locked.clear();
+
     std::string str_tmp;
     std::list< std::string > list_tmp;
     std::list< std::string >::iterator it_tmp;
@@ -227,6 +234,8 @@ void read_list_urls( JDLIB::ConfLoader& cf, const std::string& id_urls, const st
         for( ; it_tmp != list_tmp.end(); ++it_tmp ) if( !(*it_tmp).empty() ) list_urls.push_back( (*it_tmp));
     }
 
+    if( ! list_urls.size() ) return;
+
     str_tmp = cf.get_option( id_locked, "");
     if( ! str_tmp.empty() ){
         list_tmp = MISC::split_line( str_tmp );
@@ -549,6 +558,10 @@ void SESSION::init_session()
 
     popupmenu_shown = false;
 
+    img_fit_mode = cf.get_option( "img_fit_mode", IMG_FIT_NORMAL );
+
+    dir_select_favorite = cf.get_option( "dir_select_favorite", "" );
+
     // WM 判定
     // TODO: 環境変数で判定できない場合の判定方法を考える
     win_manager = WM_UNKNON;
@@ -747,7 +760,11 @@ void SESSION::save_session()
 
         << "img_dir_dat_save = " << img_dir_dat_save << std::endl
         << "img_dir_img_save = " << img_dir_img_save << std::endl
-        << "dir_draft = " << dir_draft << std::endl;
+        << "dir_draft = " << dir_draft << std::endl
+
+        << "img_fit_mode = " << img_fit_mode << std::endl
+
+        << "dir_select_favorite = " << dir_select_favorite << std::endl;
 
     CACHE::save_rawdata( CACHE::path_session(), oss.str() );
 
@@ -1162,3 +1179,28 @@ void SESSION::remove_live( const std::string& url )
 #endif
 }
 
+
+// 画像のfitモード
+const int SESSION::get_img_fit_mode()
+{
+    return img_fit_mode;
+}
+
+
+void SESSION::toggle_img_fit_mode()
+{
+    if( img_fit_mode == IMG_FIT_NORMAL ) img_fit_mode = IMG_FIT_WIDTH;
+    else img_fit_mode = IMG_FIT_NORMAL;
+}
+
+
+// お気に入り挿入ダイアログで最後に保存したディレクトリ名
+const std::string& SESSION::get_dir_select_favorite()
+{
+    return dir_select_favorite;
+}
+
+void SESSION::set_dir_select_favorite( const std::string& dir )
+{
+    dir_select_favorite = dir;
+}
