@@ -7,17 +7,22 @@
 #include "window.h"
 #include "view.h"
 #include "dragnote.h"
+#include "msgdiag.h"
 
 #include "dbtree/interface.h"
 
 #include "jdlib/miscutil.h"
 #include "jdlib/miscgtk.h"
 
+#include "history/historymanager.h"
+#include "history/viewhistoryitem.h"
+
+#include "control/controlutil.h"
+#include "control/controlid.h"
+
 #include "command.h"
 #include "session.h"
 #include "global.h"
-#include "controlutil.h"
-#include "controlid.h"
 #include "updatemanager.h"
 
 
@@ -34,7 +39,8 @@ Admin::Admin( const std::string& url )
     : m_url( url ),
       m_win( NULL ),
       m_notebook( NULL ),
-      m_focus( false )
+      m_focus( false ),
+      m_use_viewhistory( false )
 {
     m_notebook = new DragableNoteBook();
 
@@ -47,8 +53,7 @@ Admin::Admin( const std::string& url )
     m_notebook->sig_tab_menu().connect( sigc::mem_fun( *this, &Admin::slot_tab_menu ) );
 
     // D&D
-    m_notebook->sig_drag_begin().connect( sigc::mem_fun(*this, &Admin::slot_drag_begin ) );
-    m_notebook->sig_drag_end().connect( sigc::mem_fun(*this, &Admin::slot_drag_end ) );
+    m_notebook->sig_drag_data_get().connect( sigc::mem_fun(*this, &Admin::slot_drag_data_get ) );
 
     m_list_command.clear();
 }
@@ -97,10 +102,10 @@ void Admin::setup_menu( const bool enable_checkupdate )
     m_action_group = Gtk::ActionGroup::create();
     m_action_group->add( Gtk::Action::create( "Quit", "Quit" ), sigc::mem_fun( *this, &Admin::slot_close_tab ) );
 
-    m_action_group->add( Gtk::ToggleAction::create( "LockTab", "タブをロックする(_L)", std::string(), false ),
+    m_action_group->add( Gtk::ToggleAction::create( "LockTab", "タブをロックする(_K)", std::string(), false ),
                          sigc::mem_fun( *this, &Admin::slot_lock ) );
 
-    m_action_group->add( Gtk::Action::create( "Close_Tab_Menu", "複数のタブを閉じる(_M)" ) );
+    m_action_group->add( Gtk::Action::create( "Close_Tab_Menu", "複数のタブを閉じる(_L)" ) );
     m_action_group->add( Gtk::Action::create( "CloseOther", "他のタブ(_O)" ), sigc::mem_fun( *this, &Admin::slot_close_other_tabs ) );
     m_action_group->add( Gtk::Action::create( "CloseLeft", "左←のタブ(_L)" ), sigc::mem_fun( *this, &Admin::slot_close_left_tabs ) );
     m_action_group->add( Gtk::Action::create( "CloseRight", "右→のタブ(_R)" ), sigc::mem_fun( *this, &Admin::slot_close_right_tabs ) );
@@ -119,6 +124,12 @@ void Admin::setup_menu( const bool enable_checkupdate )
 
     m_action_group->add( Gtk::Action::create( "Preference", "プロパティ(_P)..."), sigc::mem_fun( *this, &Admin::show_preference ) );
 
+    // 戻る、進む
+    m_action_group->add( Gtk::Action::create( "PrevView", "PrevView"),
+                         sigc::bind< int >( sigc::mem_fun( *this, &Admin::back_clicked_viewhistory ), 1 ) );
+    m_action_group->add( Gtk::Action::create( "NextView", "NextView"),
+                         sigc::bind< int >( sigc::mem_fun( *this, &Admin::forward_clicked_viewhistory ), 1 ) );
+
     m_ui_manager = Gtk::UIManager::create();    
     m_ui_manager->insert_action_group( m_action_group );
 
@@ -191,14 +202,29 @@ void Admin::setup_menu( const bool enable_checkupdate )
 
     m_move_menu = Gtk::manage( new Gtk::Menu() );
 
+    // 進む、戻る
+    Glib::RefPtr< Gtk::Action > act;
+    act = m_action_group->get_action( "PrevView" );
+    act->set_accel_group( m_ui_manager->get_accel_group() );
+    item = Gtk::manage( act->create_menu_item() );
+    m_move_menu->append( *item );
+
+    act = m_action_group->get_action( "NextView" );
+    act->set_accel_group( m_ui_manager->get_accel_group() );
+    item = Gtk::manage( act->create_menu_item() );
+    m_move_menu->append( *item );
+
+    m_move_menu->append( *Gtk::manage( new Gtk::SeparatorMenuItem() ) );
+
     // 先頭、最後に移動
-    item = Gtk::manage( new Gtk::MenuItem( "先頭に移動(_H)", true ) );
+    item = Gtk::manage( new Gtk::MenuItem( "先頭のタブに移動(_H)", true ) );
     m_move_menu->append( *item );
     item->signal_activate().connect( sigc::mem_fun( *this, &Admin::tab_head ) );
 
-    item = Gtk::manage( new Gtk::MenuItem( "最後に移動(_T)", true ) );
+    item = Gtk::manage( new Gtk::MenuItem( "最後のタブに移動(_T)", true ) );
     m_move_menu->append( *item );
     item->signal_activate().connect( sigc::mem_fun( *this, &Admin::tab_tail ) );
+
     m_move_menu->append( *Gtk::manage( new Gtk::SeparatorMenuItem() ) );
 
     item  = Gtk::manage( new Gtk::MenuItem( "移動" ) );
@@ -429,12 +455,17 @@ void Admin::exec_command()
 
     // 前回終了時の状態を回復
     if( command.command == "restore" ){
-        restore();
+        restore( ( command.arg1 == "only_locked" ) );
     }
 
     // 移転などでホストの更新
-    else if( command.command == "update_host" ){
-        update_host( command.url, command.arg1 );
+    else if( command.command == "update_url" ){
+        update_url( command.url, command.arg1 );
+    }
+
+    // 板名更新
+    else if( command.command == "update_boardname" ){
+        update_boardname( command.url );
     }
 
     // viewを開く
@@ -445,7 +476,7 @@ void Admin::exec_command()
     // arg1 にはdatファイルを空白で区切って指定する
     //
     else if( command.command == "open_list" ){
-        open_list( command.arg1 );
+        open_list( command );
     }
     else if( command.command == "switch_view" ){
         switch_view( command.url );
@@ -484,9 +515,12 @@ void Admin::exec_command()
     else if( command.command == "update_finish" ){
         update_finish( command.url );
     }
+
+    // 全ロック解除
     else if( command.command == "unlock_views" ){
         unlock_all_view( command.url );
     }
+
     else if( command.command == "close_view" ){
         if( command.arg1 == "closeall" ) close_all_view( command.url );
         else close_view( command.url );
@@ -537,7 +571,9 @@ void Admin::exec_command()
     // ステータス表示
     // アクティブなviewから依頼が来たらコアに渡す
     else if( command.command == "set_status" ){
-        set_status( command.url, command.arg1, false );
+
+        bool force = ( command.arg2 == "force" );
+        set_status( command.url, command.arg1, force );
     }
 
     // マウスジェスチャ
@@ -568,14 +604,39 @@ void Admin::exec_command()
         return;
     }
 
+    // ツールバーの検索ボックスをフォーカス
+    else if( command.command == "focus_toolbar_search" ){
+        focus_toolbar_search();
+    }
+
     // ツールバー表示切り替え
     else if( command.command == "toggle_toolbar" ){
         toggle_toolbar();
     }
 
+    // ツールバーのアドレスを更新
+    else if( command.command == "update_toolbar_url" ){
+        m_notebook->update_toolbar_url( command.url, command.arg1 );
+    }
+
+    // ツールバーのラベルを更新
+    else if( command.command == "redraw_toolbar" ){
+        redraw_toolbar();
+    }
+
     // ツールバーボタン表示更新
-    else if( command.command == "update_toolbar" ){
-        update_toolbar();
+    else if( command.command == "update_toolbar_button" ){
+        update_toolbar_button();
+    }
+
+    // 検索バー表示
+    else if( command.command == "open_searchbar" ){
+        open_searchbar();
+    }
+
+    // 検索バー非表示
+    else if( command.command == "close_searchbar" ){
+        close_searchbar();
     }
 
     // タブ表示切り替え
@@ -603,10 +664,65 @@ void Admin::exec_command()
         if( view ) view->show_preference();
     }
 
-    // 個別のコマンド処理
-    else command_local( command );
-}
+    // View履歴:戻る
+    else if( command.command == "back_viewhistory" ){
+        back_viewhistory( command.url, atoi( command.arg1.c_str() ) );
+    }
 
+    // View履歴:進む
+    else if( command.command == "forward_viewhistory" ){
+        forward_viewhistory( command.url, atoi( command.arg1.c_str() ) );
+    }
+
+    // View履歴削除
+    else if( command.command == "clear_viewhistory" ){
+        clear_viewhistory();
+    }
+
+    // オートリロード開始
+    else if( command.command == "start_autoreload" ){
+
+        int mode = AUTORELOAD_ON;
+        if( command.arg1 == "once" ) mode = AUTORELOAD_ONCE;
+        int sec = atoi( command.arg2.c_str() );
+
+        set_autoreload_mode( command.url, mode, sec );
+    }
+
+    // オートリロード停止
+    else if( command.command == "stop_autoreload" ){
+
+        set_autoreload_mode( command.url, AUTORELOAD_NOT, 0 );
+    }
+
+    else{
+
+        // ツールバー関係
+        SKELETON::View* view = get_view( command.url );
+
+        if( view && command.command == "toolbar_exec_search" ) view->exec_search();
+        else if( view && command.command == "toolbar_operate_search" ) view->operate_search( command.arg1 );
+        else if( view && command.command == "toolbar_set_search_query" ) view->set_search_query( command.arg1 );
+        if( view && command.command == "toolbar_up_search" ) view->up_search();
+        if( view && command.command == "toolbar_down_search" ) view->down_search();
+
+        else if( view && command.command == "toolbar_write" ) view->write();
+        else if( view && command.command == "toolbar_reload" ) view->reload();
+        else if( view && command.command == "toolbar_stop" ) view->stop();
+        else if( view && command.command == "toolbar_close_view" ) view->close_view();
+        else if( view && command.command == "toolbar_delete_view" ) view->delete_view();
+        else if( view && command.command == "toolbar_set_favorite" ) view->set_favorite();
+
+        // ロック/アンロック
+        else if( view && command.command == "toolbar_lock_view" ){
+            if( view->is_locked() ) unlock( get_current_page() );
+            else lock( get_current_page() );
+        }
+
+        // 子クラス別のコマンド処理
+        else command_local( command );
+    }
+}
 
 
 // リストで与えられたページをタブで連続して開く
@@ -614,19 +730,25 @@ void Admin::exec_command()
 // 連続してリロードかけるとサーバに負担をかけるので、オフラインで開いて
 // タイミングをずらしながらリロードする
 //
-void Admin::open_list( const std::string& str_list )
+void Admin::open_list( const COMMAND_ARGS& command_list )
 {
+#ifdef _DEBUG
+    std::cout << "Admin::open_list " << m_url << std::endl;
+#endif
+
+    const std::string& str_list = command_list.arg1;
+
     std::list< std::string > list_url = MISC::split_line( str_list );
     if( list_url.empty() ) return;
 
     int waittime = 0;
-    bool online = SESSION::is_online();
+    const bool online = SESSION::is_online();
 
     std::list< std::string >::iterator it = list_url.begin();
     for( ; it != list_url.end(); ++it, waittime += AUTORELOAD_MINSEC ){
 
         // 各admin別の引数をセット
-        COMMAND_ARGS command_arg = get_open_list_args( ( *it ) );
+        COMMAND_ARGS command_arg = get_open_list_args( ( *it ), command_list );
 
         // 共通の引数をセット
         command_arg.command = "open_view";
@@ -635,6 +757,10 @@ void Admin::open_list( const std::string& str_list )
         command_arg.arg2 = "false";  // 既に開いているかチェック
         command_arg.arg3 = "noswitch";  // タブを切り替えない
 
+#ifdef _DEBUG
+        std::cout << "url = " << command_arg.url << std::endl;
+#endif
+
         open_view( command_arg );
 
         // 一番最初のページは普通にオンラインで開く
@@ -653,12 +779,12 @@ void Admin::open_list( const std::string& str_list )
 
 
 //
-// 移転などでviewのホスト名を更新
+// 移転などでviewのurlを更新
 //
-void Admin::update_host( const std::string& oldhost, const std::string& newhost )
+void Admin::update_url( const std::string& url_old, const std::string& url_new )
 {
 #ifdef _DEBUG
-    std::cout << "Admin::update_host " << oldhost << " -> " << newhost << std::endl;
+    std::cout << "Admin::update_url " << url_old << " -> " << url_new << std::endl;
 #endif
 
     int pages = m_notebook->get_n_pages();
@@ -666,12 +792,34 @@ void Admin::update_host( const std::string& oldhost, const std::string& newhost
 
         for( int i = 0; i < pages; ++i ){
             SKELETON::View* view = dynamic_cast< View* >( m_notebook->get_nth_page( i ) );
-            if( view && view->get_url().find( oldhost ) == 0 ) view->update_host( newhost );
+            if( view && view->get_url().find( url_old ) == 0 ) view->update_url( url_old, url_new );
         }
     }
 }
 
 
+//
+// urlを含むviewの板名を更新
+//
+void Admin::update_boardname( const std::string& url )
+{
+#ifdef _DEBUG
+    std::cout << "Admin::update_boardname url = " << url << std::endl;
+#endif
+
+    int pages = m_notebook->get_n_pages();
+    if( pages ){
+
+        for( int i = 0; i < pages; ++i ){
+            SKELETON::View* view = dynamic_cast< View* >( m_notebook->get_nth_page( i ) );
+            if( view && view->get_url().find( url ) == 0 ) view->update_boardname();
+        }
+
+        redraw_toolbar();
+    }
+}
+
+
 //
 // URLやステータスを更新
 //
@@ -738,13 +886,16 @@ void Admin::open_view( const COMMAND_ARGS& command )
     view = create_view( command );
     if( !view ) return;
 
-
     int page = m_notebook->get_current_page();
-    bool open_tab = (  page == -1 || command.arg1 == "true" || command.arg1 == "right" || command.arg1 == "left"
+    bool open_tab = (  page == -1
+                       || command.arg1 == "true" || command.arg1 == "right" || command.arg1 == "left"
                        || command.arg3 == "auto" // オートモードの時もタブで開く
                        || is_locked( page )
         );
 
+    // ツールバー表示
+    if( page == -1 ) show_toolbar();
+
     // タブで表示
     if( open_tab ){
 
@@ -759,6 +910,17 @@ void Admin::open_view( const COMMAND_ARGS& command )
 
         // 最後に表示
         else m_notebook->append_page( command.url, *view );
+
+        if( m_use_viewhistory ){
+
+            if( m_last_closed_url.empty() ) HISTORY::get_history_manager()->create_viewhistory( view->get_url() );
+
+            // 直前に閉じたViewの履歴を次に開いたViewに引き継ぐ
+            else{
+                HISTORY::get_history_manager()->append_viewhistory( m_last_closed_url, view->get_url() );    
+                m_last_closed_url = std::string();
+            }
+        }
     }
 
     // 開いてるviewを消してその場所に表示
@@ -766,17 +928,27 @@ void Admin::open_view( const COMMAND_ARGS& command )
 #ifdef _DEBUG
         std::cout << "replace page\n";
 #endif
+        // タブ入れ替え
         m_notebook->insert_page( command.url, *view, page );
 
         m_notebook->remove_page( page + 1 );
 
-        if( current_view ) delete current_view;
+        if( current_view ){
+
+            std::string url_current = current_view->get_url();
+            delete current_view;
+
+            if( m_use_viewhistory ) HISTORY::get_history_manager()->append_viewhistory( url_current, view->get_url() );    
+        }
     }
 
     m_notebook->show_all();
     view->show();
     view->show_view();
 
+    // ツールバーの情報更新
+    m_notebook->set_current_toolbar( view->get_id_toolbar(), view );
+
     if( ! ( command.arg3 == "noswitch" ) ){
         switch_admin();
         set_current_page( m_notebook->page_num( *view ) );
@@ -787,6 +959,34 @@ void Admin::open_view( const COMMAND_ARGS& command )
 }
 
 
+//
+// ツールバーの検索ボックスをフォーカス
+//
+void Admin::focus_toolbar_search()
+{
+    m_notebook->focus_toolbar_search();
+}
+
+
+//
+// ツールバー表示更新
+//
+void Admin::redraw_toolbar()
+{
+    SKELETON::View* view = get_current_view();
+    if( view ) m_notebook->set_current_toolbar( view->get_id_toolbar(), view );
+}
+
+
+//
+// ツールバーのボタン表示更新
+//
+void Admin::update_toolbar_button()
+{
+    m_notebook->update_toolbar_button();
+    redraw_toolbar();
+}
+
 
 //
 // ビュー切り替え
@@ -964,6 +1164,13 @@ void Admin::close_view( SKELETON::View* view )
 
     m_notebook->remove_page( page );
 
+    if( m_use_viewhistory ){
+
+        // 直前に閉じたViewの履歴を次に開いたViewに引き継ぐ
+        if( ! m_last_closed_url.empty() ) HISTORY::get_history_manager()->delete_viewhistory( m_last_closed_url );
+        m_last_closed_url = view->get_url();
+    }
+
     delete view;
 
 #ifdef _DEBUG
@@ -975,6 +1182,9 @@ void Admin::close_view( SKELETON::View* view )
 #ifdef _DEBUG
         std::cout << "empty\n";
 #endif
+
+        m_notebook->hide_toolbar();
+
         CORE::core_set_command( "empty_page", m_url );
     }
 }
@@ -996,7 +1206,10 @@ void Admin::unlock_all_view( const std::string& url )
     for( ; it != list_view.end(); ++it ){
 
         SKELETON::View* view = ( *it );
-        if( view && view->is_locked() ) view->unlock();
+        if( view && view->is_locked() ){
+            view->unlock();
+            redraw_toolbar();
+        }
     }
 }
 
@@ -1059,14 +1272,22 @@ void Admin::update_item( const std::string& url,  const std::string& id )
     std::cout << "Admin::update_item : " << url << " " << id << std::endl;
 #endif
 
-    SKELETON::View* view = get_view( url );
-    if( view ) view->update_item( id );
+    std::list< SKELETON::View* > list_view = get_list_view();
+    std::list< SKELETON::View* >::iterator it = list_view.begin();
+    for( ; it != list_view.end(); ++it ){
+        SKELETON::View* view = ( *it );
+        if( view ) view->update_item( url, id );
+    }
+
 }
 
 
 //
 // ビューに更新終了を知らせる
 //
+// update_view はデータロード時などの更新途中でも呼び出されるが
+// update_finish は最後に一度だけ呼び出される
+//
 void Admin::update_finish( const std::string& url )
 {
 #ifdef _DEBUG
@@ -1117,6 +1338,7 @@ void Admin::set_url( const std::string& url, const std::string& url_show, const
 //
 // ステータス表示
 //
+// virtual
 void Admin::set_status( const std::string& url, const std::string& stat, const bool force )
 {
     if( m_win ) m_win->set_status( stat );
@@ -1222,7 +1444,15 @@ void Admin::set_tablabel( const std::string& url, const std::string& str_label )
 #endif
 
     SKELETON::View* view = get_view( url );
-    if( view ) m_notebook->set_tab_fulltext( str_label, m_notebook->page_num( *view ) );
+    if( view ){
+
+        m_notebook->set_tab_fulltext( str_label, m_notebook->page_num( *view ) );
+
+        // View履歴のタイトルも更新
+        if( m_use_viewhistory ){
+            HISTORY::get_history_manager()->replace_current_title_viewhistory( view->get_url(), str_label );
+        }
+    }
 }
 
 
@@ -1241,33 +1471,6 @@ void Admin::relayout_all()
 }
 
 
-//
-// ツールバー表示切り替え
-//
-void Admin::toggle_toolbar()
-{
-    std::list< SKELETON::View* > list_view = get_list_view();
-    std::list< SKELETON::View* >::iterator it = list_view.begin();
-    for( ; it != list_view.end(); ++it ){
-        SKELETON::View* view = ( *it );
-        if( view ) view->toggle_toolbar();
-    }
-}
-
-
-//
-// ツールバーボタン表示更新
-//
-void Admin::update_toolbar()
-{
-    std::list< SKELETON::View* > list_view = get_list_view();
-    std::list< SKELETON::View* >::iterator it = list_view.begin();
-    for( ; it != list_view.end(); ++it ){
-        SKELETON::View* view = ( *it );
-        if( view ) view->update_toolbar();
-    }
-}
-
 
 //
 // タブ表示切り替え
@@ -1284,6 +1487,10 @@ void Admin::toggle_tab()
 //
 void Admin::toggle_icon( const std::string& url )
 {
+#ifdef _DEBUG
+    std::cout << "Admin::toggle_icon url = " << url << std::endl;
+#endif
+
     SKELETON::View* view = get_view( url );
     if( view ){
 
@@ -1295,7 +1502,10 @@ void Admin::toggle_icon( const std::string& url )
         // オートリロードモードでロード待ち
         else if( view->get_autoreload_mode() != AUTORELOAD_NOT ) iconname = "loading_stop";
 
-        // 更新あり   
+        // 更新可能
+        else if( view->is_check_update() ) iconname = "update";
+
+        // 更新済み
         else if( view->is_updated() ){
 
             // タブがアクティブの時は通常アイコンを表示
@@ -1303,12 +1513,12 @@ void Admin::toggle_icon( const std::string& url )
             else iconname = "updated";
         }
 
-        // 更新チェック済み
-        else if( view->is_check_update() ) iconname = "update";
-
         // 古い
         else if( view->is_old() ) iconname = "old";
 
+#ifdef _DEBUG
+        std::cout << "name = " << iconname << std::endl;
+#endif
         int id = view->get_icon( iconname );
         get_notebook()->set_tabicon( iconname, get_notebook()->page_num( *view ), id );
     }
@@ -1350,6 +1560,9 @@ bool Admin::set_autoreload_mode( const std::string& url, int mode, int sec )
 //
 View* Admin::get_view( const std::string& url )
 {
+    SKELETON::View* view = get_current_view();
+    if( view && view->get_url() == url ) return view;
+
     int pages = m_notebook->get_n_pages();
     if( pages ){
 
@@ -1468,14 +1681,17 @@ void Admin::slot_switch_page( GtkNotebookPage*, guint page )
     std::cout << "Admin::slot_switch_page : " << m_url << " page = " << page << std::endl;
 #endif
 
-    // タブのアイコンを通常に戻して再描画
     SKELETON::View* view = dynamic_cast< View* >( m_notebook->get_nth_page( page ) );
     if( view ){
 #ifdef _DEBUG
         std::cout << "url = " << view->get_url() << std::endl;
 #endif
-        toggle_icon( view->get_url() );
 
+        // ツールバー表示切り替え
+        m_notebook->set_current_toolbar( view->get_id_toolbar(), view );
+
+        // タブのアイコンを通常に戻して再描画
+        toggle_icon( view->get_url() );
         view->redraw_view();
         if( m_focus ){
             update_status( view, false );
@@ -1543,6 +1759,8 @@ void Admin::slot_tab_menu( int page, int x, int y )
     Glib::RefPtr< Gtk::Action > act;
     m_clicked_page = -1; // メニューのactive状態を変えたときにslot関数が呼び出されるのをキャンセル
 
+    SKELETON::View* view =  dynamic_cast< View* >( m_notebook->get_nth_page( page ) );
+
     // ロック
     act = m_action_group->get_action( "LockTab" );
     if( page >= 0 && act ){
@@ -1564,6 +1782,20 @@ void Admin::slot_tab_menu( int page, int x, int y )
         else act->set_sensitive( true );
     }
 
+    // 進む、戻る
+    if( view ){
+        act = m_action_group->get_action( "PrevView" );
+        if( act ){
+            if( HISTORY::get_history_manager()->can_back_viewhistory( view->get_url(), 1 ) ) act->set_sensitive( true );
+            else act->set_sensitive( false );
+        }
+        act = m_action_group->get_action( "NextView" );
+        if( act ){
+            if( HISTORY::get_history_manager()->can_forward_viewhistory( view->get_url(), 1 ) ) act->set_sensitive( true );
+            else act->set_sensitive( false );
+        }
+    }
+
     m_clicked_page = page;
 
     Gtk::Menu* popupmenu = dynamic_cast< Gtk::Menu* >( m_ui_manager->get_widget( "/popup_menu" ) );
@@ -1593,7 +1825,7 @@ void Admin::slot_tab_menu( int page, int x, int y )
 
         // コメント更新
         Gtk::Label* label = dynamic_cast< Gtk::Label* >( m_vec_movemenu_items[ MAX_TABS ]->get_child() );
-        if( label ) label->set_text( "移動 [ タブ数 " + MISC::itostr( pages ) +" ]" );
+        if( label ) label->set_text_with_mnemonic( "移動 [ タブ数 " + MISC::itostr( pages ) +" ](_M)" );
 
         m_move_menu->show_all();
 
@@ -1894,13 +2126,19 @@ const bool Admin::is_locked( const int page )
 void Admin::lock( const int page )
 {
     SKELETON::View* view =  dynamic_cast< View* >( m_notebook->get_nth_page( page ) );
-    if( view ) return view->lock();
+    if( view ){
+        view->lock();
+        redraw_toolbar();
+    }
 }
 
 void Admin::unlock( const int page )
 {
     SKELETON::View* view =  dynamic_cast< View* >( m_notebook->get_nth_page( page ) );
-    if( view ) return view->unlock();
+    if( view ){
+        view->unlock();
+        redraw_toolbar();
+    }
 }
 
 // プロパティ表示
@@ -1909,3 +2147,167 @@ void Admin::show_preference()
     SKELETON::View* view =  dynamic_cast< View* >( m_notebook->get_nth_page( m_clicked_page ) );
     if( view ) view->show_preference();
 }
+
+//
+// View履歴:戻る
+//
+bool Admin::back_viewhistory( const std::string& url, const int count )
+{
+    return back_forward_viewhistory( url, true, count );
+}
+
+
+void Admin::back_clicked_viewhistory( const int count )
+{
+    if( m_clicked_page < 0 ) return;
+
+    SKELETON::View* view =  dynamic_cast< View* >( m_notebook->get_nth_page( m_clicked_page ) );
+    if( ! view ) return;
+    
+    m_notebook->set_current_page( m_clicked_page );
+    back_forward_viewhistory( view->get_url(), true, count );
+}
+
+
+//
+// View履歴進む
+//
+bool Admin::forward_viewhistory( const std::string& url, const int count )
+{
+    return back_forward_viewhistory( url, false, count );
+}
+
+
+void Admin::forward_clicked_viewhistory( const int count )
+{
+    if( m_clicked_page < 0 ) return;
+
+    SKELETON::View* view =  dynamic_cast< View* >( m_notebook->get_nth_page( m_clicked_page ) );
+    if( ! view ) return;
+
+    m_notebook->set_current_page( m_clicked_page );
+    back_forward_viewhistory( view->get_url(), false, count );
+}
+
+
+//
+// 戻る、進む
+//
+bool Admin::back_forward_viewhistory( const std::string& url, const bool back, const int count )
+{
+    if( ! m_use_viewhistory ) return false;
+
+#ifdef _DEBUG
+    std::cout << "Admin::back_forward_viewhistory back = " << back
+              << "count = " << count << " tab = " << url << std::endl;
+#endif
+
+    SKELETON::View* view = get_view( url );
+    if( view ){
+
+        if( view->is_locked() ) return false;
+
+        const HISTORY::ViewHistoryItem* historyitem;
+
+        if( back ) historyitem = HISTORY::get_history_manager()->back_viewhistory( url, count, false );
+        else historyitem = HISTORY::get_history_manager()->forward_viewhistory( url, count, false );
+
+        if( historyitem && ! historyitem->url.empty() ){
+#ifdef _DEBUG
+            std::cout << "open : " << historyitem->url << std::endl;
+#endif
+
+            // 既にタブで開いている場合
+            if( get_view( historyitem->url) ){
+
+                // 次のviewを開けるかチェック
+                bool enable_next = false;
+                if( back ) enable_next = HISTORY::get_history_manager()->can_back_viewhistory( url, count +1 );
+                else enable_next = HISTORY::get_history_manager()->can_forward_viewhistory( url, count +1 );
+
+                SKELETON::MsgDiag mdiag( get_win(), historyitem->title + "\n\nは既にタブで開いています",
+                                         false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE );
+                mdiag.add_button( "タブを開く(_T)", Gtk::RESPONSE_YES );
+                if( enable_next ) mdiag.add_button( "次を開く(_N)", Gtk::RESPONSE_NO );
+                mdiag.add_button( Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL );
+
+                int ret = mdiag.run();
+                mdiag.hide();
+
+                switch( ret ){
+
+                    case Gtk::RESPONSE_YES:
+                        switch_view( historyitem->url );
+                        return false;
+
+                    case Gtk::RESPONSE_NO:
+                        return back_forward_viewhistory( url, back, count + 1 );
+
+                    default:
+                        break;
+                }
+
+                return false;
+            }
+
+            // 検索ビューなど、back/forwardしたときに view のurlが変わることがあるので
+            // view履歴内のURLを置き換える必要がある。そこで Admin::open_view() 中の
+            // append_viewhistory() を実行しないでここで View履歴の現在位置を変更する
+            // Admin::Open_view()も参照すること
+            const bool use_history = get_use_viewhistory();
+            set_use_viewhistory( false );
+
+            if( use_history ){
+                if( back ) HISTORY::get_history_manager()->back_viewhistory( url, count, true );
+                else HISTORY::get_history_manager()->forward_viewhistory( url, count, true );
+            }
+
+            COMMAND_ARGS command_arg = url_to_openarg( historyitem->url, false, false );
+            open_view( command_arg );
+
+            // URLが変わっていたらview履歴内のURLも更新
+            if( use_history ){
+
+                SKELETON::View* current_view = get_current_view();
+                if( current_view && current_view->get_url() != historyitem->url ){
+                    HISTORY::get_history_manager()->replace_current_url_viewhistory( historyitem->url, current_view->get_url() );
+                    redraw_toolbar();
+                }
+
+                set_use_viewhistory( true );
+            }
+
+            return true;
+        }
+    }
+
+    return false;
+}
+
+
+//
+// View履歴削除
+//
+void Admin::clear_viewhistory()
+{
+    if( ! m_use_viewhistory ) return;
+
+    std::list< SKELETON::View* > list_view = get_list_view();
+    std::list< SKELETON::View* >::iterator it = list_view.begin();
+    for( ; it != list_view.end(); ++it ){
+        SKELETON::View* view = ( *it );
+        if( view ){
+            HISTORY::get_history_manager()->delete_viewhistory( (*it)->get_url() );
+            HISTORY::get_history_manager()->create_viewhistory( (*it)->get_url() );
+
+            int page = m_notebook->page_num( *(*it) );
+            std::string str_label = m_notebook->get_tab_fulltext( page );
+            HISTORY::get_history_manager()->replace_current_title_viewhistory( (*it)->get_url(), str_label );
+        }
+    }
+
+    if( ! m_last_closed_url.empty() ) HISTORY::get_history_manager()->delete_viewhistory( m_last_closed_url );
+    m_last_closed_url = std::string();
+
+    update_toolbar_button();
+}
